import { select, csv, max } from 'd3';

var width = 960 / 2;
var height = 195;

//Create SVG element
var svg = d3
    .select('body')
    .select('#Plot1')
    .append('svg')
    .attr('width', width)
    .attr('height', height),
  svg2 = d3
    .select('body')
    .select('#Plot2')
    .append('svg')
    .attr('width', width + 40)
    .attr('height', height + 40),
  svg3 = d3
    .select('body')
    .select('#Plot3')
    .append('svg')
    .attr('width', width + 40)
    .attr('height', height + 40),
  svg4 = d3
    .select('body')
    .select('#Plot4')
    .append('svg')
    .attr('width', width + 40)
    .attr('height', height + 40)
    .append('g')
    .attr(
      'transform',
      'translate(' +
        width / 2 +
        ',' +
        height / 2 +
        ')'
    );

function updateDC(data) {
  var ordScale = d3
    .scaleOrdinal()
    .domain(
      data.map(function (d) {
        return d[0];
      })
    )
    .range(d3.schemeCategory10);
  var pie = d3.pie().value(function (d) {
    return d[1];
  });

  var arc = svg4
    .selectAll('arc')
    .data(pie(data))
    .enter();

  var path = d3
    .arc()
    .outerRadius(Math.min(width, height) / 2)
    .innerRadius(50);

  arc
    .append('path')
    .attr('d', path)
    .attr('fill', function (d) {
      return ordScale(d.data[0]);
    })
    .on('mouseover', function (d) {
      var x = d3.event.pageX,
        y = d3.event.pageY;
      d3.select('#tooltip')
        .style('display', 'block')
        .style('left', d3.event.pageX + 'px')
        .style('top', d3.event.pageY - 30 + 'px')
        .select('#value')
        .html(
          'Sub-Category: ' +
            d.data[0] +
            '<br>' +
            'Sales: ' +
            d.data[1]
        );
    });

  arc.exit().remove();
}

let usaProjection = d3
  .geoAlbersUsa()
  .scale(400)
  .translate([width / 2, height / 2]);
let pathGenerator = d3
  .geoPath()
  .projection(usaProjection);
const geoJsonUrl =
  'https://gist.githubusercontent.com/spiker830/e0d1b7950ced31369c903bed0cead7b1/raw/702c72e0ca5a1be95f84a50a58cfa6d4d6400f3f/us_features.json';

// X axis
var x = d3
  .scaleBand()
  .range([40, width])
  .padding(0.2);
var xAxis = svg2
  .append('g')
  .attr(
    'transform',
    `translate(${20}, ${+height})`
  );

// Add Y axis
var y = d3.scaleLinear().range([height, 0]);
var yAxis = svg2
  .append('g')
  .attr('transform', 'translate(' + 60 + ',0)');

// Scatter plot
// X axis
var xSP = d3.scaleTime().range([40, width]);

var xAxisSP = svg3
  .append('g')
  .attr('transform', `translate(0, ${+height})`);

// Add Y axis
var ySP = d3.scaleLinear().range([height, 10]);
var yAxisSP = svg3
  .append('g')
  .attr('transform', 'translate(' + 40 + ',0)');

function updateSP(data) {
  var colorSub = d3
    .scaleOrdinal()
    .domain(
      data.map(function (d) {
        return d['Sub-Category'];
      })
    )
    .range([
      '#5E4FA2',
      '#3288BD',
      '#66C2A5',
      '#ABDDA4',
      '#E6F598',
      '#FFFFBF',
      '#FEE08B',
      '#FDAE61',
      '#F46D43',
      '#D53E4F',
      '#9E0142',
    ]);

  xSP.domain([
    moment('01/01/2015', 'DD/MM/YYYY').toDate(),
    moment('01/01/2019', 'DD/MM/YYYY').toDate(),
  ]);
  xAxisSP
    .transition()
    .duration(1000)
    .call(d3.axisBottom(xSP))
    .selectAll('text')
    .style('text-anchor', 'end')
    .attr('dx', '-1em')
    .attr('dy', '-0.5em')
    .attr('transform', 'rotate(-60)');

  ySP.domain([0, max(data, (d) => +d.Sales)]);
  yAxisSP
    .transition()
    .duration(1000)
    .call(d3.axisLeft(ySP));

  var u = svg3.selectAll('circle').data(data);

  u.enter()
    .append('circle')
    // .style("opacity", 0.2)
    .on('mouseover', function (d) {
      d3.select(this)
        .attr('r', '10')
        .attr('fill', 'orange');
      var x = d3.event.pageX,
        y = d3.event.pageY;
      d3.select('#tooltip')
        .style('display', 'block')
        .style('left', d3.event.pageX + 'px')
        .style('top', d3.event.pageY - 30 + 'px')
        .select('#value')
        .html(
          'Customer: ' +
            d['Customer Name'] +
            '<br>' +
            'Product: ' +
            d['Product Name']
        );
    })
    .on('mouseout', function (d) {
      d3.select(this)
        .attr('r', '6')
        .attr('fill', function (d) {
          var col = colorSub(d['Sub-Category']);
          return col;
        });
      d3.select('#tooltip').style(
        'display',
        'none'
      );
    })
    .merge(u)
    .transition()
    .duration(1000)
    .attr('cx', function (d) {
      return xSP(
        moment(
          d['Order Date'],
          'DD/MM/YYYY'
        ).toDate()
      );
    })
    .attr('cy', function (d) {
      return ySP(d.Sales);
    })
    .attr('r', '6')
    .attr('fill', function (d) {
      var col = colorSub(d['Sub-Category']);
      return col;
    })
    .attr('stroke', 'black');

  u.exit().remove();
}

// Bar plot
function updateBP(data) {
  // Using d3.v6 for some cool features.
  // Constitute a dataset likes {Cat, Sales}
  var salesByCategory = d3.rollup(
    data,
    (v) => d3.sum(v, (d) => d.Sales),
    (d) => d['Category']
  );

  // Get the biggest cat.
  var biggestCat = d3.least(
    d3.rollup(
      data,
      (v) => d3.sum(v, (d) => d.Sales),
      (d) => d.Category
    ),
    ([, sum]) => -sum
  );

  x.domain(
    data.map(function (d) {
      return d['Category'];
    })
  );
  xAxis
    .transition()
    .duration(1000)
    .call(d3.axisBottom(x))
    .selectAll('text')
    .style('text-anchor', 'end')
    .attr('dx', '-1em')
    .attr('dy', '-0.5em')
    .attr(
      'transform',
      'translate(' + 20 + ',' + 10 + ')'
    );

  y.domain([0, biggestCat[1]]);
  yAxis
    .transition()
    .duration(1000)
    .call(d3.axisLeft(y));
  var u = svg2
    .selectAll('rect')
    .data(Array.from(salesByCategory.entries()));

  u.enter()
    .append('rect')
    .on('mouseover', function (d) {
      d3.select(this).attr('fill', 'red');
      var x = d3.event.pageX,
        y = d3.event.pageY;
      d3.select('#tooltip')
        .style('display', 'block')
        .style('left', d3.event.pageX + 10 + 'px')
        .style('top', d3.event.pageY - 30 + 'px')
        .select('#value')
        .html(
          'Category: ' +
            d[0] +
            '<br>' +
            'Sales: ' +
            d[1]
        );
    })
    .on('mouseout', function (d) {
      d3.select(this).attr('fill', '#69b3a2');
      d3.select('#tooltip').style(
        'display',
        'none'
      );
    })
    .on('click', function (d) {
      svg2
        .selectAll('rect')
        .style('fill', '#69b3a2');

      d3.select(this).style('fill', 'orange');

      var donutData = data.filter(function (l) {
        return l['Category'] == d[0];
      });

      var salesBySub = d3.rollup(
        donutData,
        (v) => d3.sum(v, (d) => d.Sales),
        (d) => d['Sub-Category']
      );
      updateDC(Array.from(salesBySub.entries()));

      updateSP(
        data.filter(function (l) {
          return l['Category'] == d[0];
        })
      );
    })
    .merge(u)
    .transition()
    .duration(1000)
    .attr('x', function (d) {
      return x(d[0]);
    })
    .attr('y', function (d) {
      return y(d[1]);
    })
    .attr('width', x.bandwidth())
    .attr('height', function (d) {
      return height - y(d[1]);
    })
    .attr('fill', '#69b3a2');
}

// Export csv data
csv('data.csv').then((data) => {
  data.forEach((d, i) => {
    d.Delay = Math.floor(
      (moment(
        d['Ship Date'],
        'DD/MM/YYYY'
      ).toDate() -
        moment(
          d['Order Date'],
          'DD/MM/YYYY'
        ).toDate()) /
        (1000 * 60 * 60 * 24)
    );
  });
  updateBP(data);
});
